' Pillenwarner
' erzeugt alle 24h bzw. alle 12h ein Blinksignal, das quittiert werden muss
'
' Hardware:
' Start der Uhr durch Reset- Taster
' Takt durch externen Quarzoszillator mit 3,6864 MHz an PB3
' LED an PB4
' Quittungstaster an PB2
' Umschaltung 24h- / 12h- Modus durch Schalter an PB0
'
' 12h -> 43200 sec.  24h -> 86400 sec

' AtTiny mu auf Quarzbetrieb geflasht werden ( LFuse = 0xFD )
'
' Quarztakt wird zur Laufzeit mit Clkpr auf 3,686400 / 64 herabgesetzt
'
'
'----------------------------------------------------------

$regfile = "attiny25.dat"
$crystal = 57600                                            ' 3686400 / 64


$hwstack = 32
$swstack = 16
$framesize = 16

On Timer0 Ontimer0                                          'Interrupt-Routine


' Variablen

Dim T As Long                                               ' Sekunden
Dim M As Long                                               ' Anzahl Sekunden fr Modus 12h / 24h
Dim A As Byte                                               ' Alarm
Dim N As Byte                                               ' Zhlvariable



' Register


Acsr.7 = 1                                                  ' Analog-Komparator ausschalten, spart etwas Strom
Sreg.7 = 1                                                  ' Global Interrupt Enable
Gimsk.5 = 1                                                 ' PCint enable
Pcmsk.2 = 1                                                 ' PCint PB2 enable
Didr0 = &B0001_1010                                         ' Digitaleingnge PB1, PB3, PB4 abschalten
Ddrb = &B0000_0010                                          ' alle Pins Eingang bis auf PB1
Portb = &B0000_0101                                         ' Pullup-Widerstand PB0 und PB2 anschalten


' Alias

Led Alias Portb.1
Taster Alias Pinb.2
Schalter Alias Pinb.0


' Konstanten
'
' Sekunden * Minuten * Stunden * Tage - 1 Sekunde
Const M0 = 60 * 60 * 24 - 1                                 ' 86399 = 24h - 1s
Const M1 = 60 * 60 * 12 - 1                                 ' 43199 = 12h - 1s


' Timer 0 fr 1 Interrupt / sec vorbereiten:

Config Timer0 = Timer , Prescale = 256
Enable Timer0
Start Timer0


Clkpr = &B10000000                                          ' nderung Clockteiler vorbereiten
Clkpr = &B00000110                                          ' 3686400 / 64
Clkpr = &B10000110

'----------------------------------------------------------




' Hauptprogramm

If Schalter = 0 Then                                        ' 12h- Modus, wenn Schalter auf "ON"
   M = M1
   Else
   M = M0
End If

 T = 7200                                                   ' Uhr einmalig 2 h vorstellen, damit Alarm knftig 2h frher
                                                            ' als Reset- Zeitpunkt


' Hauptschleife

Do


   N = N + 1

   If N > 7 Then                                            ' Lebenszeichen; alle 8 Sek. kurzes Blinken
      N = 0
      Led = 1
      Waitms 20
      Led = 0
   End If

'     If T > 29 Then                                         ' Testbetrieb 30 sek statt 24h / 12h
    If T > M Then                                           ' Realbetrieb 24h/12h sind um
        T = 0                                               ' Uhr rcksetzen
        A = A + 1                                           ' Alarmbyte
    End If


    If Taster = 0 Then                                      ' Rcksetzen Alarm, wenn Quittungstaster gedrckt
        A = 0
    End If

    If A = 1 Then                                           ' Einfachblinken bei Alarm
        Led = 1
        Waitms 100
        Led = 0
    End If

    If A > 1 Then                                           ' Doppelblinken, wenn "alter Alarm" nicht quittiert wurde
       Led = 1
       Waitms 100
       Led = 0
       Waitms 200
       Led = 1
       Waitms 100
       Led = 0
    End If

    Idle


Loop






Ontimer0:                                                   'Timer1-Overflow-Interrupt-Routine

   Tcnt0 = 31                                               ' Preset auf 31, damit er 225 Takte bis Ueberlauf zhlt
   Incr T

Return